"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeferredPromise = void 0;
const Errors_js_1 = require("../common/Errors.js");
const environment_js_1 = require("../environment.js");
/**
 * Creates an returns a promise along with the resolve/reject functions.
 *
 * If the promise has not been resolved/rejected within the `timeout` period,
 * the promise gets rejected with a timeout error.
 *
 * @internal
 */
function createDeferredPromise({ message, timeout = 5000, } = {}) {
    if (environment_js_1.DEFERRED_PROMISE_DEBUG_TIMEOUT > 0 && !timeout) {
        timeout = environment_js_1.DEFERRED_PROMISE_DEBUG_TIMEOUT;
    }
    let isResolved = false;
    let isRejected = false;
    let resolver = (_) => { };
    let rejector = (_) => { };
    const taskPromise = new Promise((resolve, reject) => {
        resolver = resolve;
        rejector = reject;
    });
    const timeoutId = message
        ? setTimeout(() => {
            isRejected = true;
            rejector(new Errors_js_1.TimeoutError(message));
        }, timeout)
        : undefined;
    return Object.assign(taskPromise, {
        resolved: () => {
            return isResolved;
        },
        finished: () => {
            return isResolved || isRejected;
        },
        resolve: (value) => {
            if (timeoutId) {
                clearTimeout(timeoutId);
            }
            isResolved = true;
            resolver(value);
        },
        reject: (err) => {
            clearTimeout(timeoutId);
            isRejected = true;
            rejector(err);
        },
    });
}
exports.createDeferredPromise = createDeferredPromise;
//# sourceMappingURL=DeferredPromise.js.map